//** legend **

function drawLegend() {
	var legendRectSize=18,
	legendSpacing=4;

	var color = d3.scale.ordinal()
		.domain(["AbstractToken","CYPHER Keyword","FUDGE Keyword","Identifier or Number","Punctuation", "Operator","Unknown","ErrorToken"])
		.range(["lightblue","lightblue","violet","lightgreen","lightgrey", "lightcoral","#FAA689", "orange"]);
		
	var color2 = d3.scale.ordinal()
		.domain(["AbstractToken","CYPHER Keyword","FUDGE Keyword","Identifier or Number","Punctuation", "Operator","Unknown","ErrorToken"])
		.range(["black","#1d75b3","purple","green", "grey", "maroon","#FA4002","black"]);
		
	var legend = d3.select('svg')
		.append("g")
		.selectAll("g")
		.data(color.domain())
		.enter()
		.append('g')
		  .attr('class', 'legend')
		  .attr('transform', function(d, i) {
			var height = legendRectSize+10;
			var x = 1;
			var y = 1+i * height;
			return 'translate(' + x + ',' + y + ')';
		});

	legend.append('rect')
		.attr('width', legendRectSize)
		.attr('height', legendRectSize)
		.style('fill', color)
		.style('stroke', color2)
		.style('stroke-width',2);

	legend.append('text')
		.attr('x', legendRectSize + legendSpacing)
		.attr('y', legendRectSize - legendSpacing)
		.text(function(d) { return d; });
}


// ************** Generate the tree diagram	 *****************
var margin = {top: 20, right: 120, bottom: 20, left: 120},
width = 5000 - margin.right - margin.left,
height = 600 - margin.top - margin.bottom;

var i = 0,
	duration = 750,
	root;

var tree = d3.layout.tree()
	.size([height, width]);

var diagonal = d3.svg.diagonal()
	.projection(function(d) { return [d.y, d.x]; });

var svg = d3.select("body").append("svg")
	.attr("width", width + margin.right + margin.left)
	.attr("height", height + margin.top + margin.bottom)
  .append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");
	
d3.json("data.json", function (json) {
	root = json[0];
	root.x0 = height/2;
	root.y0 = 0;
	update(root);
	drawLegend();
});
 
d3.select(self.frameElement).style("height", "500px");


function update(source) {

  // Compute the new tree layout.
  var nodes = tree.nodes(root).reverse(),
	  links = tree.links(nodes);

  // Normalize for fixed-depth.
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  // Update the nodes…
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

  // Enter any new nodes at the parent's previous position.
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; })
	  .on("click", click);

  nodeEnter.append("circle")
	  .attr("r", function(d){ return d.value; })
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });

  nodeEnter.append("text")
	  .attr("x", function(d) { return ((d.children || d._children) || d.level == "orange" ) ? (d.value + 4) * -1 : d.value + 4 })
	  .attr("y", function(d) { return ((d.children || d._children) || d.level == "orange" )  ? (d.value + 4) * -1 : 0 })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return ((d.children || d._children) || d.level == "orange" ) ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);

  // Transition nodes to their new position.
  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  nodeUpdate.select("circle")
	  .attr("r", 10)
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });
	

  nodeUpdate.select("text")
	  .style("fill-opacity", 1)
	  .style("fill", function(d){ return d.type; });

  // Transition exiting nodes to the parent's new position.
  var nodeExit = node.exit().transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + source.y + "," + source.x + ")"; })
	  .remove();

  nodeExit.select("circle")
	  .attr("r", 1e-6)
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });
 
  nodeExit.select("text")
	  .style("fill-opacity", 1e-6);

  // Update the links…
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

  // Enter any new links at the parent's previous position.
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .style("stroke", function (d) { return d.target.level;})
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

  // Transition links to their new position.
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);

  // Transition exiting nodes to the parent's new position.
  link.exit().transition()
	  .duration(duration)
	  .attr("d", function(d) {
		var o = {x: source.x, y: source.y};
		return diagonal({source: o, target: o});
	  })
	  .remove();

  // Stash the old positions for transition.
  nodes.forEach(function(d) {
	d.x0 = d.x;
	d.y0 = d.y;
  });
}

// Toggle children on click.
function click(d) {
  if (d.children) {
	d._children = d.children;
	d.children = null;
  } else {
	d.children = d._children;
	d._children = null;
  }
  update(d);
  drawLegend();
}