package LexicalAnalyser;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * Lexer
 * Classe réalisant l'analyse lexicale d'une requête FUDGE
 * String -> ArrayList<Token>
 *
 */
public class Lexer {
	/**
	 * Méthode réalisant la tokenization d'une requête FUDGE
	 * String -> ArrayList<Token>
	 * @param _input une chaîne de caractères (requête FUDGE)
	 * @return une liste d'élements (Token)
	 * @see Token 
	 */
	public static ArrayList<Token> tokenize(String _input) {
		ArrayList<Token> tokens = new ArrayList<Token>();
		StringBuffer sBuffer = new StringBuffer();
		
		_input = _input.trim().replaceAll("([dD][iI][sS][tT][iI][nN][cC][tT])","DISTINCT");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][dD][eE][sS][cC])","DEFINEDESC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][aA][sS][cC])","DEFINEASC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][qQ][rR][dD][eE][sS][cC])","DEFINEQRDESC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][qQ][rR][aA][sS][cC])","DEFINEQRASC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][qQ][aA][dD][eE][sS][cC])","DEFINEQADESC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE][qQ][aA][aA][sS][cC])","DEFINEQAASC");
		_input = _input.trim().replaceAll("([dD][eE][fF][iI][nN][eE])","DEFINE");
		_input = _input.trim().replaceAll("([aA][sS])","AS");
		_input = _input.trim().replaceAll("([iI][nN])","IN");
		_input = _input.trim().replaceAll("([iI][sS])","IS");
		_input = _input.trim().replaceAll("([wW][hH][eE][rR][eE])","WHERE");
		_input = _input.trim().replaceAll("([mM][aA][tT][cC][hH])","MATCH");
		_input = _input.trim().replaceAll("([hH][aA][vV][iI][nN][gG])","HAVING");
		
		for(TokenType tType : TokenType.values()) {
			sBuffer.append(String.format("|(?<%s>%s)", tType.name(), tType.pattern));
		}
		
		Pattern tPatterns = Pattern.compile(new String(sBuffer.substring(1)));

		Matcher matcher = tPatterns.matcher(_input);
		while(matcher.find()) {
			for(TokenType tType: TokenType.values()) {
				if(matcher.group(tType.name())!=null) {
					if(tType!=TokenType.WHITESPACE && tType!=TokenType.COMMENTS && tType!=TokenType.COMMENTSLN) {
							tokens.add(new Token(tType, matcher.group(tType.name())));
					}
				}
			}
		}
		
		return tokens;
	}
}
