package LexicalAnalyser;

import SyntacticalAnalyser.Syntaxer;
import SyntacticalAnalyser.TokenTree;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * TokenType
 * Enum pour la spécification des différents types de Tokens
 * Il existe deux types de TokenType :
 * <ul>
 * <li>les types "concrets" définis par une expression réguliere</li>
 * <li>les types abstraits</li>
 * </ul>
 * Les types concrets contiennent des informations sur les sous-chaînes de caractères de la requête FUDGE, ils sont manipulés lors de l'analyxe lexico-syntaxique
 * Les types abstraits servent à définir les noeuds abstrait de l'arbre syntaxique abstrait.
 * @see Lexer
 * @see Syntaxer
 * @see TokenTree
 *
 */
public enum TokenType {
	COMMENTSLN("//.*\n"),
	COMMENTS("//.*$"),
	WHITESPACE("[ \t\f\r\n]+"),
	//Cypher Keywords
	MATCH("MATCH"),
	WHERE("WHERE"),
	WITH("WITH"),
	REDUCE("REDUCE"),
	RETURN("RETURN"),
	OPTIONAL("OPTIONAL"),
	DELETE("DELETE"),
	DISTINCT("DISTINCT"),
	//SUGAR Keywords
	kwDEFINEASC("DEFINEASC"),
	kwDEFINEDESC("DEFINEDESC"),
	kwDEFINE("DEFINE"),
	kwAS("AS"),
	kwIN("IN"),
	kwIS("IS"),
	kwLENGTH("Length"),
	kwST("ST"),
	//Punctuations
	pLBRACKET("\\("),
	pRBRACKET("\\)"),
	pCOMMA(","),
	pDLBRACKET("\\[\\["),
	pDRBRACKET("\\]\\]"),
	//Operators
	oVERTICAL("\\|"),
	oSTAR("\\*"),
	oPLUS("\\+"),
	oUNION("\u222A"),
	oDOT("\\."),
	//Identifiers & Numbers
	ID("[a-zA-Z]+"),
	NUMBER("[0-9]+(\\.[0-9]+)?"),
	UNKNOWN("[\\p{ASCII}]"),
	//
	FuzzyDefineBlock,
	FuzzyRelationshipBlock,
	FuzzyConditionsBlock,
	CypherBlock,
	FuzzyRegularExpressionBlock,
	DefineBlock,
	DefineAscOrDescBlock,
	QueryBlock,
	RelationshipBlock,
	ConditionBlock,
	ErrorToken,
	
	Expression,
	Group,
	GroupOrId,
	StarOrPlusExpression;
	
	public final String pattern;
	
	private TokenType(String _pattern) {
		pattern=_pattern;
	}
	
	private TokenType() {
		pattern=null;
	}
}