package SyntacticalAnalyser;

import java.util.ArrayList;

import LexicalAnalyser.Token;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * SyntaxException
 * Classe pour la spécification des différentes erreurs de syntaxes lors de l'analyse syntaxique
 * @see Syntaxer
 *
 */
public class SyntaxException extends Exception {
	private static final long serialVersionUID = 1L;
	public static ArrayList<Token> tokens;
	public static int charCounter = 0;
	private static String errorValue;
	
	private String 
		cBlock = "Cypher Block",
		
		qDef = "'DEFINE'",
		qDefAsc = "'DEFINEASC'",
		qDefDes = "'DEFINEDESC'",
		qIs = "'IS'",
		qIn = "'IN'",
		qSt = "'ST'",
		qLen = "'Length'",
		
		qDLBra ="'[['",
		qDRBra ="']]'",
		
		mWasExpected = "was expected.",
		
		mExp = "A Fuzzy Regular Expression "+mWasExpected,
		mId = "An Identifier "+mWasExpected+" Regular Expression Syntax : [a-zA-Z]+",
		mNum = "An Number "+mWasExpected+" Regular Expression Syntax : [0-9]+",
		mCom = "',' "+mWasExpected,
		mLBra = "'(' "+mWasExpected,
		mRBra = "')' "+mWasExpected,
		mDef = qDef+" "+mWasExpected,
		mAs = "'AS' "+mWasExpected,
		mDAOD = qDefAsc+" or "+qDefDes+" "+mWasExpected,
		mIs = qIs+" "+mWasExpected,
		mDLBra = qDLBra+" "+mWasExpected,
		mDRBra = qDRBra+" "+mWasExpected;
	
	public SyntaxException(int i, TokenTree tree) {
		String 
			errorInfo = "[Error "+i+"] ",
			charInfo = "@Char n°",
			queryInfo = "In query : ",
			arrowInfo = "           ";
		
		Syntaxer.addErrorToken(tree);
		
		switch (i) {
			//Empty
			case 0:errorInfo+="Query is Empty.";break;
					
			//Fuzzy Define Block
			case 101:errorInfo+=qIn+", "+qDef+", "+qDefAsc+" or "+qDefDes+" "+mWasExpected;break;
			case 102:errorInfo+=cBlock+" "+qIs+", "+cBlock+" "+qDLBra+" or "+cBlock+" "+mWasExpected;break;
			
			//Define Block
			case 111:errorInfo+=mDef;break;//TODO
			case 112:errorInfo+=mId;break;
			case 113:errorInfo+=mAs;break;
			case 114:errorInfo+=mLBra;break;
			case 115:errorInfo+=mNum;break;
			case 116:errorInfo+=mCom;break;
			case 117:errorInfo+=mNum;break;
			case 118:errorInfo+=mCom;break;
			case 119:errorInfo+=mNum;break;
			case 120:errorInfo+=mCom;break;
			case 121:errorInfo+=mNum;break;
			case 122:errorInfo+=mRBra;break;
			
			//Define Asc Or Desc Block
			case 131:errorInfo+=mDAOD;break;//TODO
			case 132:errorInfo+=mId;break;
			case 133:errorInfo+=mAs;break;
			case 134:errorInfo+=mLBra;break;
			case 135:errorInfo+=mNum;break;
			case 136:errorInfo+=mCom;break;
			case 137:errorInfo+=mNum;break;
			case 138:errorInfo+=mRBra;break;
			
			//Fuzzy Relationship Block
			case 201:errorInfo+=mDLBra;break;//TODO
			case 202:errorInfo+=mExp;break;
			case 203:errorInfo+=qLen+" or "+qSt+" "+mWasExpected;break;
			case 204:errorInfo+=mIs;break;
			case 205:errorInfo+=mId;break;
			case 206:errorInfo+=mDRBra;break;
			case 207:errorInfo+=cBlock+" "+qIs+", "+cBlock+" "+qDLBra+" or "+cBlock+" "+mWasExpected;break;
			
			//Expression
			case 210:errorInfo+="210";break; //TODO
			case 211:errorInfo+="211";break; //TODO
			case 212:errorInfo+="212";break; //TODO
			case 213:errorInfo+="213";break; //TODO
			case 214:errorInfo+="214";break; //TODO
			case 215:errorInfo+="215";break; //TODO
			case 216:errorInfo+="216";break; //TODO
			case 217:errorInfo+="217";break; //TODO
			case 218:errorInfo+="218";break; //TODO
			case 219:errorInfo+="219";break; //TODO
			case 220:errorInfo+="220";break; //TODO
			case 221:errorInfo+="221";break; //TODO
			case 222:errorInfo+="222";break; //TODO
			
			//Fuzzy Conditions Block
			case 301:errorInfo+=mIs;break;//*
			case 302:errorInfo+=mId;break;
			case 303:errorInfo+=cBlock+" "+qIs+" or "+cBlock+" "+mWasExpected;break;
			default:errorInfo+="Unknown Syntax Error.";break;
		}
		
		if(tokens!=null) {
			for(Token token : tokens) {
				queryInfo+=token.getData()+" ";
			}
		}
		
		charInfo+=charCounter;
		
		for(int j=0;j<charCounter;j++) {
			arrowInfo+=" ";
		}
		
		arrowInfo+="^";
		
		/*FileEdit.replaceContent("<b style=\"color:red\";>"
				+errorInfo+"<br>"
				+charInfo+"<br>"
				+queryInfo.substring(0,10)+"</b><i>"
				+queryInfo.substring(10, charCounter+11)+"<b style=\"background-color: #F7950C;color:white;\">"
				+queryInfo.substring(charCounter+11,charCounter+12)+"</b>"
				+queryInfo.substring(charCounter+12, queryInfo.length())+"</i><br>");*/
		if(queryInfo.length()>12+charCounter) {
			errorValue="<b style=\"color:red\";>"
					+errorInfo+"<br>"
					+charInfo+"<br>"
					+queryInfo.substring(0,10)+"</b><i>"
					+queryInfo.substring(10, charCounter+11)+"<b style=\"background-color: #F7950C;color:white;\">"
					+queryInfo.substring(charCounter+11,charCounter+12)+"</b>"
					+queryInfo.substring(charCounter+12, queryInfo.length())+"</i><br>";
		}
		
		System.err.println(errorInfo+"\n"+charInfo+"\n"+queryInfo+"\n"+arrowInfo);
	}

	public static void init(ArrayList<Token> _tokens) {
		charCounter=0;
		tokens=_tokens;
		errorValue="";
		//FileEdit.deleteFile();
	}

	public static String getErrorValue() {
		return errorValue;
	}
}
