package SyntacticalAnalyser;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import LexicalAnalyser.AbstractToken;
import LexicalAnalyser.Token;
import LexicalAnalyser.TokenType;
import Tools.Debug;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * Syntaxer
 * Classe réalisant l'analyse syntaxique d'une requête FUDGE
 * ArrayList<Token> -> TokenTree
 *
 */
public class Syntaxer {
	private static Token currentToken;
	private static Iterator<Token> tokens;
	
	/**
	 * Méthode réalisant la génération d'un arbre syntaxique abstrait à partir d'une liste de tokens
	 * @param _tokens une liste de tokens issus de la requête FUDGE
	 * @return un arbre syntaxique abstrait
	 * @see TokenTree
	 */
	public static TokenTree generateATS(ArrayList<Token> _tokens) {
		Debug.afficheMethod(Syntaxer.class);
		
		currentToken=null;
		
		//Exception Handler
		SyntaxException.init(_tokens);
		
		TokenTree ats = new TokenTree(new AbstractToken(TokenType.QueryBlock));
		try {
			if(_tokens != null) {
				tokens = _tokens.iterator();
				next();
				TokenTree fdTree = new TokenTree(new AbstractToken(TokenType.FuzzyDefineBlock));
				ats.addSon(fdTree);
				fuzzyDefineBlock(fdTree, ats);
			} else {
				throw new SyntaxException(0,ats);
			}
		} catch (SyntaxException e) {
			e.printStackTrace();
		}
		
		writeJsonData(ats);
		
		return ats;
	}
	
	private static void writeJsonData(TokenTree ats) {
		try {
			File file = new File("fabs/data.json");
			FileWriter fw = new FileWriter(file, false);
			fw.write("["+ats.toJsonString(0)+"]");
			fw.close();
			//PrintWriter writer = new PrintWriter("src/main/webapp/fabs/data.json","UTF-8");
			//writer.println("["+ats.toJsonString(0)+"]");
			//writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void fuzzyDefineBlock(TokenTree fdTree, TokenTree ats) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		if(checkToken(TokenType.kwDEFINE) || checkToken(TokenType.kwDEFINEASC) || checkToken(TokenType.kwDEFINEDESC)) {
			while(!checkToken(TokenType.kwIN)) {
				if(checkToken(TokenType.kwDEFINE)) {
					TokenTree dTree = new TokenTree(new AbstractToken(TokenType.DefineBlock)); 
					fdTree.addSon(dTree);
					defineBlock(dTree);
				} else if(checkToken(TokenType.kwDEFINEASC) || checkToken(TokenType.kwDEFINEDESC)){
					TokenTree daodTree = new TokenTree(new AbstractToken(TokenType.DefineAscOrDescBlock)); 
					fdTree.addSon(daodTree);
					defineAscOrDescBlock(daodTree);
				} else {
					throw new SyntaxException(101,fdTree);
				}
			}
			
			//IN
			addToken(fdTree);
		}
			//c NO Tokens Fudge, s'arrete à IS ou fin de liste
			//sinon erreur
		Boolean 
			fin=false,
			cypher=false;
		TokenTree cTree = new TokenTree(new AbstractToken(TokenType.CypherBlock)); 
		while(!fin) {
			if((checkToken(TokenType.kwIS) || checkToken(TokenType.pDLBRACKET) || currentToken==null) && cypher) {
				fin=true;
			} else if (currentToken!=null && checkNonKeyword()  && !checkToken(TokenType.kwIS) && !checkToken(TokenType.pDLBRACKET)) {
				if(!cypher) {
					fdTree.addSon(cTree);
					cypher=true;
				}
				addToken(cTree);
			} else {
				fin=true;
				throw new SyntaxException(102,fdTree);
			}
		}
		
		if(currentToken!=null) {
			if(checkToken(TokenType.pDLBRACKET)) {
				TokenTree frTree = new TokenTree(new AbstractToken(TokenType.FuzzyRelationshipBlock));
				
				ats.addSon(frTree);
				fuzzyRelationshipBlock(frTree,ats);
			}
			
			if(checkToken(TokenType.kwIS)) {
				TokenTree fcTree = new TokenTree(new AbstractToken(TokenType.FuzzyConditionsBlock));
				
				ats.addSon(fcTree);
				fuzzyConditionsBlock(fcTree);
			}
		}
	}
	
	private static void fuzzyRelationshipBlock(TokenTree frTree, TokenTree ats) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		
		while(!checkToken(TokenType.kwIS) && currentToken != null) {
			if(checkToken(TokenType.pDLBRACKET)) {
				TokenTree rTree = new TokenTree(new AbstractToken(TokenType.RelationshipBlock)); 
				frTree.addSon(rTree);
				relationshipBlock(rTree);
			} else {
				throw new SyntaxException(201,frTree);
			}
			
			//c NO Tokens Fudge, s'arrete IS ou fin de liste
			//sinon erreur
			Boolean 
				fin=false,
				cypher=false;
			TokenTree cTree = new TokenTree(new AbstractToken(TokenType.CypherBlock)); 
			while(!fin) {
				if(((checkToken(TokenType.kwIS) || checkToken(TokenType.pDLBRACKET) || currentToken==null) && cypher)) {
					fin=true;
				} else if (currentToken!=null && checkNonKeyword() && !checkToken(TokenType.kwIS) && !checkToken(TokenType.pDLBRACKET)) {
					if(!cypher) {
						frTree.addSon(cTree);
						cypher=true;
					}
					addToken(cTree);
				} else {
					fin=true;
					throw new SyntaxException(207,frTree);
				}
			}
		}
		if(currentToken!=null) {
			TokenTree fcTree = new TokenTree(new AbstractToken(TokenType.FuzzyConditionsBlock));
			
			ats.addSon(fcTree);
			fuzzyConditionsBlock(fcTree);
		}
	}

	private static void fuzzyConditionsBlock(TokenTree fcTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		while(currentToken != null) {
			if(checkToken(TokenType.kwIS)) {
				TokenTree coTree = new TokenTree(new AbstractToken(TokenType.ConditionBlock)); 
				fcTree.addSon(coTree);
				conditionBlock(coTree);
			} else {
				throw new SyntaxException(301,fcTree);
			}
			
			//c NO Tokens Fudge, s'arrete IS ou fin de liste
			//sinon erreur
			Boolean 
				fin=false,
				cypher=false;
			TokenTree cTree = new TokenTree(new AbstractToken(TokenType.CypherBlock)); 
			
			while(!fin) {
				if((checkToken(TokenType.kwIS) && cypher) || currentToken==null) {
					fin=true;
				} else if (currentToken!=null && checkNonKeyword() && !checkToken(TokenType.kwIS)) {
					if(!cypher) {
						fcTree.addSon(cTree);
						cypher=true;
					}
					addToken(cTree);
				} else {
					fin=true;
					throw new SyntaxException(303,fcTree);
				}
			}
		}
	}

	private static void defineBlock(TokenTree dTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		
		checkTokenAdd(TokenType.kwDEFINE,dTree,111);
		checkTokenAdd(TokenType.ID,dTree,112);
		checkTokenAdd(TokenType.kwAS,dTree,113);
		checkTokenAdd(TokenType.pLBRACKET,dTree,114);
		checkTokenAdd(TokenType.NUMBER,dTree,115);
		checkTokenAdd(TokenType.pCOMMA,dTree,116);
		checkTokenAdd(TokenType.NUMBER,dTree,117);
		checkTokenAdd(TokenType.pCOMMA,dTree,118);
		checkTokenAdd(TokenType.NUMBER,dTree,119);
		checkTokenAdd(TokenType.pCOMMA,dTree,120);
		checkTokenAdd(TokenType.NUMBER,dTree,121);
		checkTokenAdd(TokenType.pRBRACKET,dTree,122);
	}

	private static void defineAscOrDescBlock(TokenTree daodTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		
		checkTokenAdd(TokenType.kwDEFINEASC,TokenType.kwDEFINEDESC,daodTree,131);
		checkTokenAdd(TokenType.ID,daodTree,132);
		checkTokenAdd(TokenType.kwAS,daodTree,133);
		checkTokenAdd(TokenType.pLBRACKET,daodTree,134);
		checkTokenAdd(TokenType.NUMBER,daodTree,135);
		checkTokenAdd(TokenType.pCOMMA,daodTree,136);
		checkTokenAdd(TokenType.NUMBER,daodTree,137);
		checkTokenAdd(TokenType.pRBRACKET,daodTree,138);
	}

	private static void relationshipBlock(TokenTree rTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		addToken(rTree);
		
		//Expression
		if(checkToken(TokenType.ID) || checkToken(TokenType.pLBRACKET)) {
			TokenTree eTree = new TokenTree(new AbstractToken(TokenType.FuzzyRegularExpressionBlock)); 
			rTree.addSon(eTree);
			expressionBlock(eTree);
		} else {
			throw new SyntaxException(202,rTree);
		}
		
		
		if(checkToken(TokenType.oVERTICAL)) {
			addToken(rTree);
			
			if(checkToken(TokenType.kwLENGTH) || checkToken(TokenType.kwST)) {
				addToken(rTree);
			} else {
				throw new SyntaxException(203,rTree);
			}
			checkTokenAdd(TokenType.kwIS,rTree,204);
			checkTokenAdd(TokenType.ID,rTree,205);
		}
		checkTokenAdd(TokenType.pDRBRACKET,rTree,206);
	}
	
	private static void conditionBlock(TokenTree coTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		addToken(coTree);
		
		checkTokenAdd(TokenType.ID,coTree,302);
	}
	
	private static void expressionBlock(TokenTree eTree) throws SyntaxException {
		Debug.afficheMethod(Syntaxer.class);
		
		List<Integer> etatHistory = new ArrayList<Integer>();
		etatHistory.add(0);
		
		Boolean accept=false;
		
		while(!accept) {
			int lastIndex = etatHistory.size()-1;
			List<TokenTree> sons = eTree.getSons();
			int lastSonIndex = sons.size()-1;
			switch(etatHistory.get(lastIndex)) {
				case 0:
					if(checkToken(TokenType.ID)) {
						shift(etatHistory,eTree,4);
					} else if(checkToken(TokenType.pLBRACKET)) {
						shift(etatHistory,eTree,5);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Expression)) {
						changeState(etatHistory,1);
					} else if(sons.get(lastSonIndex).isNode(TokenType.GroupOrId)) {
						changeState(etatHistory,2);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Group)) {
						changeState(etatHistory,3);
					} else {
						throw new SyntaxException(210,eTree);
					}
					break;
				case 1:
					if(checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						accept=true;
					} else {
						throw new SyntaxException(211,eTree);
					}
					break;
				case 2:
					if(checkToken(TokenType.oDOT)) {
						shift(etatHistory,eTree,6);
					} else if(checkToken(TokenType.oSTAR) || 
							checkToken(TokenType.oPLUS) ||
							checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.Expression,1);
					} else {
						throw new SyntaxException(212,eTree);
					}
					break;
				case 3:
					if(checkToken(TokenType.oDOT) ||
							checkToken(TokenType.oSTAR) || 
							checkToken(TokenType.oPLUS) ||
							checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.GroupOrId,1);
					} else {
						throw new SyntaxException(213,eTree);
					}
					break;	
				case 4:
					if(checkToken(TokenType.oDOT) ||
							checkToken(TokenType.oSTAR) || 
							checkToken(TokenType.oPLUS) ||
							checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.GroupOrId,1);
					} else {
						throw new SyntaxException(214,eTree);
					}
					break;
				case 5:
					if(checkToken(TokenType.ID)) {
						shift(etatHistory,eTree,4);
					} else if(checkToken(TokenType.pLBRACKET)) {
						shift(etatHistory,eTree,5);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Expression)) {
						changeState(etatHistory,8);
					} else if(sons.get(lastSonIndex).isNode(TokenType.GroupOrId)) {
						changeState(etatHistory,2);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Group)) {
						changeState(etatHistory,3);
					} else if(sons.get(lastSonIndex).isNode(TokenType.StarOrPlusExpression)) {
						changeState(etatHistory,7);
					} else {
						throw new SyntaxException(214,eTree);
					}
					break;
				case 6:
					if(checkToken(TokenType.ID)) {
						shift(etatHistory,eTree,4);
					} else if(checkToken(TokenType.pLBRACKET)) {
						shift(etatHistory,eTree,5);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Expression)) {
						changeState(etatHistory,9);
					} else if(sons.get(lastSonIndex).isNode(TokenType.GroupOrId)) {
						changeState(etatHistory,2);
					} else if(sons.get(lastSonIndex).isNode(TokenType.Group)) {
						changeState(etatHistory,3);
					} else {
						throw new SyntaxException(216,eTree);
					}
					break;
				case 7:
					if(checkToken(TokenType.pRBRACKET)) {
						shift(etatHistory,eTree,10);
					} else {
						throw new SyntaxException(217,eTree);
					}
					break;
				case 8:
					if(checkToken(TokenType.oSTAR)) {
						shift(etatHistory,eTree,11);
					} else if(checkToken(TokenType.oPLUS)) {
						shift(etatHistory,eTree,12);
					} else {
						throw new SyntaxException(218,eTree);
					}
					break;
				case 9:
					if(checkToken(TokenType.oSTAR) || 
							checkToken(TokenType.oPLUS) ||
							checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.Expression,3);
					} else {
						throw new SyntaxException(219,eTree);
					}
					break;
				case 10:
					if(checkToken(TokenType.oDOT) ||
							checkToken(TokenType.oSTAR) || 
							checkToken(TokenType.oPLUS) ||
							checkToken(TokenType.oVERTICAL) ||
							checkToken(TokenType.pDRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.Group,3);
					} else {
						throw new SyntaxException(220,eTree);
					}
					break;
				case 11:
					if(checkToken(TokenType.pRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.StarOrPlusExpression,2);
					} else {
						throw new SyntaxException(221,eTree);
					}
					break;
				case 12:
					if(checkToken(TokenType.pRBRACKET)) {
						reduce(etatHistory,eTree,TokenType.StarOrPlusExpression,2);
					} else {
						throw new SyntaxException(222,eTree);
					}
					break;
				default:break;
			}
		}
	}
	
	private static void reduce(List<Integer> etatHistory, TokenTree currentTree, TokenType lme, int size) {
		TokenTree tree = new TokenTree(new AbstractToken(lme));
		ArrayList<TokenTree> sons =  currentTree.getSons();
		//on les rajoute dans le nouvel arbre
		for(int i=0;i<size;i++) {
			tree.addSon(sons.get(sons.size()-size+i)); // numero i
		}
		
		//On enleve les tokens de l'arbre original
		for(int i=0;i<size;i++) {
			sons.remove(sons.size()-1);
		}
		
		//Ajout du nouvel arbre
		currentTree.addSon(tree);
		
		//Reduce de l'etat
		for(int i=0;i<size;i++) {
			etatHistory.remove(etatHistory.size()-1);
		}
	}

	private static void changeState(List<Integer> etatHistory, int i) {
		etatHistory.add(i);
	}

	private static void shift(List<Integer> etatHistory, TokenTree tree, int i) {
		changeState(etatHistory,i);
		addToken(tree);
	}

	private static void addToken(TokenTree tokenTree) {
		Debug.afficheMethod(Syntaxer.class);
		if(currentToken != null) {
			tokenTree.addSon(new TokenTree(currentToken));
			SyntaxException.charCounter+=currentToken.getData().length()+1;
			//System.out.println("\tAjout de "+currentToken.toString());
			next();
		}
	}
	
	static void addErrorToken(TokenTree tokenTree) {
		Debug.afficheMethod(Syntaxer.class);
		
		tokenTree.addSon(new TokenTree(new AbstractToken(TokenType.ErrorToken)));
		System.out.println("\tAjout d'un ErrorToken");
	}
	
	private static boolean checkToken(TokenType tokenType) {
		if(currentToken != null) {
			return currentToken.isNode(tokenType);
		} 
		return false;
	}
	
	private static void checkTokenAdd(TokenType tokenType, TokenTree tree, int i) throws SyntaxException {
		if(checkToken(tokenType)) {
			addToken(tree);
		} else {
			throw new SyntaxException(i,tree);
		}
	}
	
	private static void checkTokenAdd(TokenType tokenType, TokenType tokenType2, TokenTree tree, int i) throws SyntaxException {
		if(checkToken(tokenType)||checkToken(tokenType2)) {
			addToken(tree);
		} else {
			throw new SyntaxException(i,tree);
		}
	}
	
	private static boolean checkNonKeyword() {
		return !checkToken(TokenType.kwDEFINEASC) && !checkToken(TokenType.kwDEFINE) && !checkToken(TokenType.kwDEFINEDESC)
				&& !checkToken(TokenType.kwDEFINEDESC) && !checkToken(TokenType.kwAS) && !checkToken(TokenType.kwIN)
				&& !checkToken(TokenType.kwLENGTH) && !checkToken(TokenType.kwST);
	}
	
	private static void next() {
		currentToken=null;
		if(tokens.hasNext()) {
			currentToken=tokens.next();
		}
	}

	/*public static String getAtsData(TokenTree ats) {
		return ats.toJsonString(0);
	}*/
}
