package SyntacticalAnalyser;

import java.util.ArrayList;

import LexicalAnalyser.AbstractToken;
import LexicalAnalyser.Token;
import LexicalAnalyser.TokenType;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * TokenTree
 * Classe de la structure de l'arbre syntaxique abstrait (AST) générée lors de l'analyse syntaxique
 * @see Syntaxer
 *
 */
public class TokenTree {
	public AbstractToken token;
	public ArrayList<TokenTree> sons;
	
	public TokenTree(Token _token, ArrayList<TokenTree> _sons) {
		super();
		token=_token;
		sons=_sons;
	}
	
	public TokenTree(){
		super();
		sons=new ArrayList<TokenTree>();
	}
	
	
	public TokenTree(AbstractToken abstractToken) {
		super();
		token=abstractToken;
		sons=new ArrayList<TokenTree>();
	}
	
	public String toJsonString(int i) {
		return toJsonString(null,i);
	}
	
	public String toJsonString(TokenTree tokenTree, int i) {
		String 
			t = "",
			s ="\n"+t+"{\n",
			type="",
			level="";
		
		for(int j=0;j<i;j++) {
			t+="\t";
		}
		
		s+=t+"\t"+"\"name\": \"";
		
		if(token !=null) {
			s+=token.toJsonString();
		} 
		
		s+="\",\n";
		
		s+=t+"\t"+"\"parent\": \"";
		
		if(tokenTree !=null) {
			s+=tokenTree.getToken().toJsonString();
		} else {
			s+="null";
		}
		
		s+="\",\n";
		
		s+=t+"\t"+"\"value\": 9,\n";
		
		if(token instanceof Token) {
			switch(token.getTokenType()) {
				case MATCH :case WHERE :case WITH :case REDUCE :case RETURN :case OPTIONAL :case DELETE :
				type="#1d75b3";level="lightblue";break;
				case kwDEFINEASC :case kwDEFINE :case kwDEFINEDESC :case kwAS :case kwIN :case kwIS :case kwLENGTH :case kwST :
					type="purple";level="violet";break;
				case pLBRACKET :case pRBRACKET :case pCOMMA :case pDLBRACKET :case pDRBRACKET : 
					type="grey";level="lightgrey";break;
				case oVERTICAL :case oSTAR :case oPLUS :case oUNION :case oDOT :
					type="maroon";level="lightcoral";break;
				case UNKNOWN :
					type="#FA4002";level="#FAA689";break;
				default:
					type = "green";level = "lightgreen";
			}
		} else {
			if(sons !=null && !sons.isEmpty()) {
				type="black";
				level="lightblue";
			} else {
				type="black";level="orange";
			}
		}
		
		s+=t+"\t"+"\"type\": \""+type+"\",\n";
		
		s+=t+"\t"+"\"level\": \""+level+"\"";
				
		if(sons !=null && !sons.isEmpty()) {
			s+=",\n";
			
			s+=t+"\t\"children\": [";
			int n = sons.size();
			for(TokenTree tt : sons) {
				s+=tt.toJsonString(this,i+1);
				if(n>1) {
					s+=",\n";
					n--;
				}
				s+="";
			}
			s+="\n"+t+"]\n";
		} else {
			s+="\n";
		}
		s+=t+"}";
		return s;
	}
	
	public String toString(int n) {
		String t = "";
		for(int i=0;i<n;i++) {
			t+="\t";
		}
		String s = "";
		if(token !=null) {
			s+=token.toString();
		}else {
			s+="null";
		}
		
		if(sons !=null && !sons.isEmpty()) {
			s+=" : [";
			for(TokenTree tt : sons) {
				s+="\n"+t+"\t"+tt.toString(n+1);
			}
			s+="\n"+t+"]";
		}
		return s;
	}

	public void addSon(TokenTree tokenTree) {
		sons.add(tokenTree);
	}

	public ArrayList<TokenTree> getSons() {
		return sons;
	}

	public void setSons(ArrayList<TokenTree> _sons) {
		sons = _sons;
	}

	public AbstractToken getToken() {
		return token;
	}
	public TokenType getTokenType() {
		if(token!=null) {
			return token.getTokenType();
		}
		return null;
	}
	
	public void setToken(Token _token) {
		token=_token;
	}

	public boolean isNode(TokenType tokenType) {
		return getTokenType().equals(tokenType);
	}
}
