package Translator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import LexicalAnalyser.AbstractToken;
import SyntacticalAnalyser.TokenTree;
import SyntacticalAnalyser.Syntaxer;
import Tools.Debug;

/**
 * 
 * @author Matthieu Biache
 * @version 1.0
 * <br>
 * Translator
 * Classe permettant :
 * <ul>
 * <li>La transformation d'un arbre syntaxique abstrait d'une requête FUDDGE en une requête CYPHER</li>
 * <li>La génération d'une HashMap contenant des couples (String nomVariableFlou, TrapezoidalFunction fonctionTrapezoidale) issus de la requête FUDGE</li>
 * </ul>
 * @see Syntaxer
 * @see TokenTree
 *
 */
public class Translator {
	private static ArrayList<FuzzyRelation> fuzzyRelationList;
	private static String query;
	private static Map<String, TrapezoidalFunction> trapezList;
	private static Set<String> nodeList,fudgeCalcList;
	private static String kk = null;
	
	public static Map<String, TrapezoidalFunction> getTrapezList() {
		return trapezList;
	}
	
	/**
	 * Méthode réalisant la transformation d'un arbre syntaxique abstrait d'une requête FUDDGE en une requête CYPHER 
	 * @param ats un arbre syntaxique abstrait
	 * @return une requête Cypher
	 */
	public static String simpleTranslate(TokenTree ats) {
		Debug.afficheMethod(Translator.class);
		
		fuzzyRelationList = new ArrayList<FuzzyRelation>();
		query="";
		trapezList = new HashMap<String, TrapezoidalFunction>();
		nodeList = new HashSet<String>();
		fudgeCalcList = new HashSet<String>();
		int indice=1,n;
		
		translateNode(ats);
		
		Pattern 
			p = Pattern.compile("\\([a-zA-Z0-9:]+\\)<?->?\\*[0-9]\\*<?->?\\([a-zA-Z0-9:]+\\)"),
			p2 = Pattern.compile("\\([a-zA-Z0-9]+(:|\\)|\\)$)");
		Matcher m = p.matcher(query);
		int i=0;
		while(m.find()) {
			String s = m.group(0);
			Matcher m2 = p2.matcher(s);
			FuzzyRelation fr = fuzzyRelationList.get(i);
			while(m2.find()) {
				fr.addProperty(m2.group(0).replaceAll("[\\(\\):]", ""));
			}
			String temp = m.group(0);
			System.out.println("**********:"+fr.getProperties().get(0));
			temp="fudge_p"+i+" = "+temp.replaceFirst("\\*"+i+"\\*", fr.getProperties().get(0));
			query = query.replace(m.group(0),temp);
			i++;
		}
		
		Matcher m3 = p2.matcher(query);
		while(m3.find()) {
			nodeList.add(m3.group(0).replaceAll("[\\(\\):]", ""));
		}
		
		String withClose="";
		n=nodeList.size();
		
		if(!fuzzyRelationList.isEmpty()) {
			withClose+=" WITH ";
		
			for(FuzzyRelation fr : fuzzyRelationList) {
				int k=fuzzyRelationList.indexOf(fr);
				switch(fr.getProperties().get(1)) {
				case "ST" : withClose+="DISTINCT REDUCE(min=1.0, edge IN relationships(fudge_p"+k+")| case when edge.fdegree<min then edge.fdegree else min end) AS fudge_p"+k;break;
				case "Length": withClose+="DISTINCT REDUCE(weight=0, edge IN relationships(fudge_p"+k+")| weight + 1/edge.fdegree) AS fudge_p"+k;break;
				default:break;
				}
				withClose+=",";
			}
			
			
			for(String s : nodeList) {
				withClose+=s+" AS "+s;
				//System.out.println(n+" "+indice);
				if(indice!=n) {
					withClose+=",";
				}
				indice++;
			}
		
		}
		
		if(query.contains("WHERE")) {
			query=query.replace("WHERE", withClose+" WHERE");
			if (!fuzzyRelationList.isEmpty()){
			for(FuzzyRelation fr : fuzzyRelationList) {
				int k=fuzzyRelationList.indexOf(fr);
			query+=" AND fudge_p"+k+""+kk+" ";
			}
			}
		} else {
			query+=withClose;
			if (!fuzzyRelationList.isEmpty()){
			for(FuzzyRelation fr : fuzzyRelationList) {
				int k=fuzzyRelationList.indexOf(fr);
			query+=" WHERE fudge_p"+k+""+kk+" ";
			}
			}
		}
		
		if(nodeList!=null && !nodeList.isEmpty()) {
			query+=" RETURN DISTINCT ";
		}
		
		indice=1;
		for(String s : nodeList) {
			query+=s;
			
			if(indice!=n) {
				query+=",";
			}
			indice++;
		}
		
		for(String s : fudgeCalcList) {
			//System.out.println("fudgeCalcList "+s);
			query+=","+s;
		}
		
		
		
		/*for(FuzzyRelation fr : fuzzyRelationList) {
			for(String s : fr.getProperties()) {
				System.out.print("fuzzyRelationList"+s);
			}
		}*/
		
    	String query1 = cleaning(query);
		query=query1;
		return query;
	}
	
	
	private static void translateNode(TokenTree tree) {
		Debug.afficheMethod(Translator.class);
		//System.out.println("\tTN "+tree.getTokenType());
		switch(tree.getTokenType()) {
			case QueryBlock:
			case FuzzyDefineBlock:
			case FuzzyRelationshipBlock:
			case FuzzyConditionsBlock:
			case FuzzyRegularExpressionBlock :
			case Expression:
			case GroupOrId:
			case Group :
			case StarOrPlusExpression:
				translateSons(tree);break;
			case DefineAscOrDescBlock:
			case DefineBlock:
				translateDefineBlock(tree);break;
			case RelationshipBlock:
				translateRelationshipBlock(tree);break;
			case ConditionBlock:
				translateConditionBlock(tree);break;
			case kwIN:break;
			case CypherBlock:
				translateSonsAsString(tree);break;
			case pLBRACKET:
			case pRBRACKET:
			case oSTAR:
			case oPLUS:
			case ID :
				query+=tree.getToken().toRawString();
				break;
			default:break;
		}
	}

	private static void translateRelationshipBlock(TokenTree tree) {
		int index=fuzzyRelationList.size();
		ArrayList<TokenTree> sons=tree.getSons();
		ArrayList<String> tokenList=getTokenList(tree);
		query+="*"+index+"*";
		FuzzyRelation fr = new FuzzyRelation();
		fr.addProperty("[:"+translateFuzzyRegExBlock(sons.get(1))+"]");
		fr.addProperty(tokenList.get(3));
		fr.addProperty(tokenList.get(5));
		String fuzzyTerm=tokenList.get(5);
			try {
			 kk=trapezList.get(fuzzyTerm).translator_query_test(fuzzyTerm, "fudge_p"+index);
		} catch (NoLowerBound | NoUpperBound e) {
			e.printStackTrace();
		}
		fudgeCalcList.add("fudge_p"+index+" AS fudge_calc_p"+index+"_"+tokenList.get(3)+"_"+tokenList.get(5));
		fuzzyRelationList.add(fr);
		
	}
	
	private static void translateConditionBlock(TokenTree tree) {
		Pattern p = Pattern.compile("(AND|WHERE) [a-zA-Z0-9\\.]+$");
		Matcher m = p.matcher(query);
		String patternString="";
		if(m.find()) {
			if(m.group(0).startsWith("A")) {
				patternString=m.group(0).substring(4);
			} else {
				patternString=m.group(0).substring(6);
		}	
		}
		
		
		ArrayList<String> tokenList=getTokenList(tree);
		for (int k=0; k<tokenList.size(); k++){
	   		System.out.println("TOKENS:" +tokenList.get(k));
	  	}
		
		String fuzzyTerm=tokenList.get(1);
		try {
			query+=trapezList.get(fuzzyTerm).translator_query_test(fuzzyTerm, patternString);
		} catch (NoLowerBound | NoUpperBound e) {
			e.printStackTrace();
		}
		
		fudgeCalcList.add(patternString+" AS fudge_calc_"+patternString.replaceAll("\\.", "_")+"_"+fuzzyTerm);
	}

	private static ArrayList<String> getTokenList(TokenTree tree) {
		ArrayList<TokenTree> sons=tree.getSons();
		ArrayList<String> tokenList=new ArrayList<String>();
		
		if(sons !=null) {
			Iterator<TokenTree> it = sons.iterator();
			while(it.hasNext()) {
				TokenTree son = it.next();
				AbstractToken token = son.getToken();
				tokenList.add(token.toRawString());
			}
		}
		return tokenList;
	}

	private static String translateFuzzyRegExBlock(TokenTree tree) {
		Debug.afficheMethod(Translator.class);
		//System.out.println("\tTFRE "+tree.getTokenType());
		String content="";
		switch(tree.getTokenType()) {
			case FuzzyRegularExpressionBlock :
			case Expression:
			case GroupOrId:
			case Group :
			case StarOrPlusExpression:
				content+=translateRegexSons(tree);break;
			case pLBRACKET:
			case pRBRACKET:
			case oSTAR:
			case oPLUS:
			case ID :
				content+=tree.getToken().toRawString();
				break;
			default:break;
		}
		return content;
	}
	
	private static String translateRegexSons(TokenTree tree) {
		Debug.afficheMethod(Translator.class);
		ArrayList<TokenTree> sons=tree.getSons();
		String content="";
		
		if(sons !=null) {
			Iterator<TokenTree> it = sons.iterator();
			while(it.hasNext()) {
				TokenTree son = it.next();
				content+=translateFuzzyRegExBlock(son);
			}
		}
		
		return content;
	}


	

	private static void translateDefineBlock(TokenTree tree) {
		ArrayList<TokenTree> sons=tree.getSons();
		ArrayList<String> tokenList=new ArrayList<String>();
		ArrayList<Double> parameters=new ArrayList<Double>();
		String defineType,fuzzyVariableName;
		TrapezoidalFunction trapezFunction = null;
		
		if(sons !=null) {
			Iterator<TokenTree> it = sons.iterator();
			while(it.hasNext()) {
				TokenTree son = it.next();
				AbstractToken token = son.getToken();
				tokenList.add(token.toRawString());
			}
		}
		
		defineType=tokenList.get(0);
		fuzzyVariableName=tokenList.get(1);
		
		for(int i=4;i<tokenList.size();i+=2) {
			parameters.add(Double.parseDouble(tokenList.get(i)));
		}
		
		switch(defineType) {
			case "DEFINEASC" : trapezFunction=new IncreasingTrapezoidalFunction(fuzzyVariableName,parameters.get(0),parameters.get(1));break;
			case "DEFINEDESC" :	trapezFunction=new DecreasingTrapezoidalFunction(fuzzyVariableName,parameters.get(0),parameters.get(1));break;
			case "DEFINE" : trapezFunction=new TrapezoidalFunction(fuzzyVariableName,parameters.get(0),parameters.get(1),parameters.get(2),parameters.get(3));break;
			default:;
		}
		
		trapezList.put(fuzzyVariableName,trapezFunction);
	}

	private static void translateSons(TokenTree tree) {
		Debug.afficheMethod(Translator.class);
		ArrayList<TokenTree> sons=tree.getSons();
		
		if(sons !=null) {
			Iterator<TokenTree> it = sons.iterator();
			while(it.hasNext()) {
				TokenTree son = it.next();
				translateNode(son);
			}
		}
	}
	
	private static void translateSonsAsString(TokenTree tree) {
		//Debug.afficheMethod(Translator.class);
		ArrayList<String> tokenList=getTokenList(tree);
		for(String token : tokenList) {
				//System.out.println("\tGSAS "+token.toRawString());
				switch(token) {
					case "MATCH":
					case "WHERE":
					case "AND":
					case "OPTIONAL":
					case "DELETE":query+=" "+token+" ";break;
					case ",":query+=token+" ";break;
					default:query+=token;
				}
		}
		
	}
	
	public static String cleaning(String query){
		String query1 = null;
	if(query.contains("*)")) {
		int l= query.indexOf("*)");
		query1=query.substring(0, l+1);
				query1+=query.substring(l+2,query.length());
				query=query1;
		
	} 
	String query2 = null;
	if(query.contains(":(")) {
		int k= query.indexOf(":(");
	 query2 = query.substring(0, k);
			query2+=query.substring(k+2,query.length());
		query=query2;
	} 
	
		return query;
	}
}
