package Translator;
/*
 * A trapezoidal function is defined by four positions (abscissa) : a, b, c, d
 * 
 *  1         ______
 *          /|      |\
 *         / |      | \
 *  0 ----   |      |   -------
 *        |  |      |  |
 *        a  b      c  d 
 *     
 * @author Virginie Thion (Univ. Rennes 1, ENSSAT Lannion / IRISA France)
 *
 */

public class TrapezoidalFunction {

	String name = "AnonymousTrapezoidalFunction";
	
	protected double a;
	protected double b;
	protected double c;
	protected double d;
	
	/**
	 * Basis constructor with four values a, b, c, d
	 * If a=b then the function is decreasing.
	 * If c=d the the function is increasing.
	 */

	
	
	public TrapezoidalFunction(String name, double a, double b, double c, double d){
		this.name = name;
		this.a = a;
		this.b = b;
		this.c = c;
		this.d = d;
	}
	
	public TrapezoidalFunction() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Constructor without name
	 */
	public TrapezoidalFunction(double a, double b, double c, double d){
		this.a = a;
		this.b = b;
		this.c = c;
		this.d = d;
		}
	
	public boolean is_decreasing(){
		if (a==b) {return true;} else return false;
	}
	
	public boolean is_increasing(){
		if (c==d) {return true;} else return false;
	}
	
	/**
	 * Calculating the ordinate value associated to an abscissa 
	 * in a line defined by two points it passes through
	 * @param xA, yA first point the line passes through
	 * @param xB, yB second point the line passes through
	 * @param x abscissa value for which the ordinate must be found on the line
	 */
	private double y_value(double xA, double yA, double xB, double yB, double x){
		// Line equation of the form y = mx+p
		double m = (yB-yA)/(xB-xA);
		double p = yB - m * xB;
		return m*x+p;
	}
	
	
	/**
	 * Calculating the y value of an abscissa x position
	 * @param x the abscissa position
	 * @return the y value of the input abscissa position
	 */
	public double mu(double x){
		// If the membership function is a decreasing one
		if (is_decreasing()){
			if (x<=c) {return 1;}
			else if (x>=d) {return 0;}
			else // c < x < d
				return y_value(c,1,d,0,x);
		}
		// If the membership function is an increasing one
		else if (is_increasing()){
			if (x>=b) {return 1;}
			else if (x<=a) {return 0;}
			else // a < x < b
				return y_value(a,0,b,1,x);
		}
		// Other case ie. a complete membership function in form of a bridge
		else if (x<=a) {return 0;}
			else if ((x>a) && (x<b)) {return y_value(a,0,b,1,x);}
			else if ((x>=b) && (x<=c)) {return 1;}
			else if ((x>c) && (x<d)) {return y_value(c,1,d,0,x);}
			else return 0;
	}
	
	public double mu(Long x){
		return mu(x.doubleValue());
	}	
	
	/**
	 * Return the abscissa bound from which any lower value as an ordinate of 0
	 * @return double
	 * @throws NoLowerBound 
	 */
	public double get_lower_bound() throws NoLowerBound{
		if (is_decreasing()) {throw new NoLowerBound("No Lower Bound as "+name+" is a decreasing membership function");}
		else return a;
	}
	
	/**
	 * Return the abscissa bound from which any upper value as an ordinate of 0
	 * @return double
	 * @throws NoUpperBound 
	 */
	public double get_upper_bound() throws NoUpperBound{
		if (is_increasing()) {throw new NoUpperBound("No Lower Bound as "+name+" is an increasing membership function");}
		else return d;
	}
	
	public String translator_query_test(String name,String lengthStr) throws NoLowerBound, NoUpperBound{
		if (is_increasing()) {return ">"+get_lower_bound();}
		else if (is_decreasing()) {return  "<"+get_upper_bound();}
		else {return ">"+get_lower_bound()+" AND "+lengthStr+"<"+get_upper_bound();}
		//else { return ">"+get_lower_bound();}
	}
	

	public String toString(){
		return (name + " trapezoidalFunction: "+ a +", "+ b +", "+ c +", "+ d+".");
	}
	
	public void print(){
		System.out.println(toString());
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	
	
	

}
