package org.neo4j.community.console;

import static org.neo4j.helpers.collection.MapUtil.map;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.neo4j.community.console.CypherQueryExecutor.CypherResult;
import org.neo4j.cypher.SyntaxException;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;
import org.neo4j.rest.graphdb.RestAPI;
import org.neo4j.rest.graphdb.RestAPIFacade;
import org.neo4j.rest.graphdb.query.RestCypherQueryEngine;
import org.neo4j.rest.graphdb.util.QueryResult;
import org.slf4j.Logger;

import com.google.gson.Gson;

import LexicalAnalyser.Lexer;
import LexicalAnalyser.Token;
import SyntacticalAnalyser.Syntaxer;
import SyntacticalAnalyser.TokenTree;
import Translator.DecreasingTrapezoidalFunction;
import Translator.IncreasingTrapezoidalFunction;
import Translator.Q1;
import Translator.Q2;
import Translator.Q3;
import Translator.Translator;
import Translator.TranslatorQuant;
import Translator.TrapezoidalFunction;
import spark.Request;

/**
 * @author mh
 * @since 30.05.12
 */
public class ConsoleService {
    private static final Logger LOG = org.slf4j.LoggerFactory.getLogger(ConsoleService.class);
    static final String DEFAULT_GRAPH_CYPHER =	
            "create" +
    // JOURNAL
      "(IJWS12:journal {name:'IJWS12', volume:14, when:2012}), \n " +
      "(IJAR14:journal {name:'IJAR14', volume:55, when:2014}), \n" +
      "(IJIS16:journal {name:'IJIS16', volume:30, when:2016}), \n" +
      "(IJIS10:journal {name:'IJIS10', volume:25, when:2010}), \n" +
      "(IJUFK15:journal {name:'IJUFK15', volume:21, when:2015}), \n" +
      //PAPER
      "(IJWS12p:paper {name:'IJWS12p', title:'A ...', year:2012, pages:14-44}), \n " +
      "(IJAR14p:paper {name:'IJAR14p', title:'An ...', year:2014, pages:74-83}),\n"+
      "(IJIS16p:paper {name:'IJIS16p', title:'On the ...', year:2016, pages:81-98}),\n"+
      "(IJIS10p:paper {name:'IJIS10p', title:'The ...', year:2010, pages:287-325}),\n"+
      "(IJIS10p1:paper {name:'IJIS10p1', title:'About ...', year:2010, pages:365-385}),\n"+
      "(IJUFK15p:paper {name:'IJUFK15p', title:'Some ...', year:2015, pages:127-138}),\n"+
      //AUTHOR
      "(Peter:author {name:'Peter'}), \n" +
      "(Andre:author {name:'Andre'}), \n" +
   //   "(Andreas:author {name:'Andreas'}), \n" +
      "(Maria:author {name:'Maria'}), \n" +
      "(Claudio:author {name:'Claudio'}), \n" +
      "(Bazil:author {name:'Bazil'}), \n" +
      "(Michel:author {name:'Michel'}), \n" +
      "(Susan:author {name:'Susan'}), \n" +
       // IMPACT FACTOR 
      "(IJWS12)-[:impactfactor {fdegree:1}]->(r1:impactfactor {value:1.88}), \n" +
      "(IJAR14)-[:impactfactor {fdegree:1}]->(r2:impactfactor {value:2.45}), \n" +
      "(IJIS16)-[:impactfactor {fdegree:1}]->(r3:impactfactor {value:1}), \n" +
      "(IJIS10)-[:impactfactor {fdegree:1}]->(r4:impactfactor {value:1.31}), \n" +
      "(IJUFK15)-[:impactfactor {fdegree:1}]->(r5:impactfactor {value:0.61}), \n" +   
      //DOMAIN 
      "(IJAR14)-[:dom {fdegree:0.6}]->(DB1:dom {name:'DB'}), \n" +
      "(IJWS12)-[:dom {fdegree:0.2}]->(DB2:dom {name:'DB'}), \n" +
      "(IJIS16)-[:dom {fdegree:0.4}]->(DB3:dom {name:'DB'}), \n" +
      "(IJIS10)-[:dom {fdegree:0.4}]->(DB4:dom {name:'DB'}), \n" +
      "(IJUFK15)-[:dom {fdegree:0.3}]->(DB5:dom {name:'DB'}), \n" +
      //PUBLISHED 
      "(IJWS12p)-[:pub {fdegree:1}]->(IJWS12), \n" +
      "(IJAR14p)-[:pub {fdegree:1}]->(IJAR14), \n" +
      "(IJIS16p)-[:pub {fdegree:1}]->(IJIS16), \n" +
      "(IJIS10p)-[:pub {fdegree:1}]->(IJIS10), \n" +
      "(IJIS10p1)-[:pub {fdegree:1}]->(IJIS10), \n" +
      "(IJUFK15p)-[:pub {fdegree:1}]->(IJUFK15), \n" +
     //AUTHOROF 
     // "(Andreas)-[:authorof {fdegree:0.5}]->(IJWS12p), \n" +
      "(Andre)-[:authorof {fdegree:0.5}]->(IJWS12p), \n" +
      "(Peter)-[:authorof {fdegree:0.5}]->(IJWS12p), \n" +
      "(Peter)-[:authorof {fdegree:0.2}]->(IJAR14p), \n" +
      "(Maria)-[:authorof {fdegree:0.4}]->(IJAR14p), \n" +
      "(Maria)-[:authorof {fdegree:0.6}]->(IJIS16p), \n" +
      "(Maria)-[:authorof {fdegree:0.5}]->(IJIS10p1), \n" +
      "(Claudio)-[:authorof {fdegree:0.6}]->(IJAR14p), \n" +
      "(Claudio)-[:authorof {fdegree:0.5}]->(IJIS10p1), \n" +
      "(Claudio)-[:authorof {fdegree:0.3}]->(IJUFK15p), \n" +
      "(Bazil)-[:authorof {fdegree:0.7}]->(IJIS10p), \n" +
      "(Michel)-[:authorof {fdegree:0.3}]->(IJUFK15p), \n" +
      //CONTRIBUTOR
      "(Peter)-[:contributor {fdegree:1}]->(Andre),  \n"  +
      "(Peter)-[:contributor {fdegree:0.04}]->(Maria), \n " +
      "(Maria)-[:contributor {fdegree:0.3}]->(Peter), \n " +
       "(Maria)-[:contributor {fdegree:0.25}]->(Claudio), \n " +
       "(Maria)-[:contributor {fdegree:0.06}]->(Michel), \n " +
       "(Maria)-[:contributor {fdegree:0.58}]->(Susan), \n " +
       "(Michel)-[:contributor {fdegree:1}]->(Maria), \n "  +
       "(Claudio)-[:contributor {fdegree:0.25}]->(Maria) " ;

     // Example of a FUDGE query
   //  static final String DEFAULT_QUERY = "DEFINEDESC short AS (1,4) DEFINEASC recent AS (2011,2014) IN MATCH (ar1:paper)-[:pub]->(j1), (ar2:paper)-[:pub]->(j2), (au1:author)-[:authorof]->(ar1), (au1)-[:authorof]->(ar2), (au1)-[[(contributor*)| Length is short]]->(au2) WHERE j1.name='IJAR14' AND j2.name='IJIS16' AND ar2.year is recent";
    // Example of Fuzzy Quantified
    static final String DEFAULT_QUERY = "DEFINEQRASC most AS (0.3,0.8) DEFINEASC high AS (0.5,2) DEFINEASC recent AS (2013,2016) DEFINEASC strong AS (0,1) IN MATCH (x:author)-[[authorof| ST IS strong]]->(p:paper)-[:pub]->(j:journal)-[:impactfactor]->(i:impactfactor), (j:journal)-[:dom]->(d) WHERE p.year IS recent WITH x HAVING most(p) ARE (i.value IS high AND d.name='DB') RETURN x";
  
    private GraphStorage storage;
   
    public ConsoleService() {
        createGraphStorage();
    }

    private void createGraphStorage() {
        try {
	    System.setProperty("org.neo4j.rest.read_timeout","5");
	    System.setProperty("org.neo4j.rest.connect_timeout","10");
	    String restUrlVar = System.getenv("NEO4J_REST_URL_VAR");
	    if (restUrlVar == null ) restUrlVar = "NEO4J_URL";
	    String restUrl = System.getenv(restUrlVar);

	    String login = null, password = null;
	    if (restUrl!=null) {
		login = System.getenv("NEO4J_LOGIN");
		password = System.getenv("NEO4J_PASSWORD");
		if (login == null) {
		    try {
			URL url = new URL(restUrl);
			String userInfo = url.getUserInfo();
			if (userInfo != null) {
			    login = userInfo.split(":")[0];
			    password = userInfo.split(":")[1];
			}
		    } catch (MalformedURLException e) {
			throw new RuntimeException("Invalid Neo4j-Server-URL " + restUrl);
		    }
		}

		if (!restUrl.contains("/db/data")) restUrl += "/db/data";
		final RestAPI api = new RestAPIFacade(restUrl, login, password);
		storage = new RestGraphStorage(api);
		log("Graph Storage " + restUrl + " login " + login + " " + password + " " + storage);
	    }
        } catch(Exception e) {
            LOG.error("Error creating graph storage",e);
        }
        if (storage == null) {
            storage = new MemoryGraphStorage();
        }
        log("Graph Storage " + storage);
    }

    private void log(String msg) {
        LOG.warn(msg);
    }

    // split init and query on ";\n"
    public Map<String, Object> execute(Neo4jService service, String init, String query, String version,Map<String,Object> params) {
    	Map colonneFuzzy=new HashMap<String, String>();
         
        if (version!=null) service.setVersion(version);
	boolean initial = init != null;
	if (dontInitialize(service) || init==null || init.equalsIgnoreCase("none")) init=null;
	if (query == null || query.equalsIgnoreCase("none")) query=null;
	final Map<String, Object> data = map("init", init, "query", query,"version",service.getVersion());
	long start = System.currentTimeMillis(), time = start;
	try {
	    time = trace("service", time);
	    if (init!=null) {
	    	final URL url = service.toUrl(init);
			if (url!=null) {
			    initFromUrl(service, url, "start n=node(*) match n-[r?]->() return n,r");
			    data.put("graph",service.exportToGeoff());
			} else if (service.isMutatingQuery(init)) {
			    for (String q : splitQuery(init)) {
			    	service.initCypherQuery(q);
			    }
			    data.put("graph",service.exportToGeoff());
			} else {
			    final Map<String,Object> graph = service.mergeGeoff(init);
	
			    data.put("graph", service.exportToJson(graph));
			}
	    }
	    if (initial) {
	    	service.setInitialized();
	    }
	    time = trace("graph", time);
	    CypherResult result = null;
	    if (query!=null) {
	    	result=fudgeProcess(data,query,service,time);
	    	//data.put("atsData", result.getAtsData());
			data.put("result", result.getText());
			data.put("json", result.getJson());
			data.put("plan", result.getPlan().toString());
			data.put("columns", result.getColumns());
			data.put("stats", result.getQueryStatistics());
	    }
	    time = trace("cypher", time);
	    if (!noViz(params)) {
		data.put("visualization", service.cypherQueryViz(result));
	    }
	    trace("viz", time);
	} catch (Exception e) {
	    e.printStackTrace();
	    data.put("error", e.toString());
	}
	time = trace("all", start);
	data.put("time", time-start);
	return data;
    }

    private CypherResult fudgeProcess(Map<String,Object> data, String query, Neo4jService service, long time) throws Exception {
    	CypherResult result = null;	
    	long[] 
    			startTime= new long[3],
    			stopTime=new long[3],
    			elapsedTime=new long[3];
		// Original fudge query, sent to the GUI
		data.put("query",query);
		if (!fuzzyQuantified(query))
			result=evaluateFUDGEquery(query,service, time, elapsedTime);	
		else 
			result= evaluateFQuantifiedQuery(query, service, time, elapsedTime);
		return result;
	} 
    
  //Method for evaluating a FUDGE query 
    public CypherResult evaluateFUDGEquery(String query, Neo4jService service, long time,  long [] elapsedTime) throws Exception {
    	CypherResult result = null;
    	long [] startTime= new long[3];
    	long[] stopTime=new long[3];
    	elapsedTime=new long[3];
    	//Analyse Lexicale / generation d'une liste de tokens
    	ArrayList<Token> tokens = Lexer.tokenize(query);
    	//Analyse Syntaxique, generation de l'arbre syntaxique abstrait
    	TokenTree ats = Syntaxer.generateATS(tokens);
    	String errorValue=SyntacticalAnalyser.SyntaxException.getErrorValue();
    	if(errorValue!="") {
    	throw new Exception(errorValue);
    	}
    	//data.put("ATS",ats.toString(0));	
    	/***/
    	startTime[0] = System.currentTimeMillis(); /*start of timer*/
     	query=Translator.simpleTranslate(ats);
       	stopTime[0] = System.currentTimeMillis();/*end of timer*/
    	elapsedTime[0] = stopTime[0] - startTime[0];/*Elapsed Time*/
    	System.out.println("\n===> period of translating from fudge to cypher query : "+elapsedTime[0]+"ms <====");
    	System.out.println("\n[SUGAR TRANSCRIPTOR MODULE] Generate Cypher query:\n"+query+"\n");
    	for (String q : splitQuery(query)) {
    	    // Executes the crisp cypher query and get an intermediate table of results  
    		startTime[1] = System.currentTimeMillis();/*start of timer*/
    	    result = service.cypherQuery(q);
    	    stopTime[1] = System.currentTimeMillis();  /*end of timer*/
    	    elapsedTime[1] = stopTime[1] - startTime[1];/*Elapsed Time*/
    	    String resultText = result.getText();
    	    System.out.println("\n===> period of calculating the intermediate results :"+elapsedTime[1]+"ms <===");
    	   System.out.println("\n[SUGAR TRANSCRIPTOR MODULE] Intermediate result:\n"+ resultText);
    	    long neo4jTime = trace(" including for neo4j engine",time);
    	    // Transforms the intermediate table of results into the new one
    	     Map <String,TrapezoidalFunction> trapezList = Translator.getTrapezList();
    	     // Satisfaction degree calculation
    		    startTime[2] = System.currentTimeMillis(); /*start of timer*/
    		    result.fudge_score_calculator_module(trapezList);
    		    stopTime[2] = System.currentTimeMillis(); /*end of timer*/
    		    elapsedTime[2] = stopTime[2] - startTime[2]; /*Elapsed Time*/
    		    String FinalResultText = result.getText();
    		    System.out.println("\n===> period of calculating the score :"+elapsedTime[2]+"ms <===");
    		    System.out.println("\n[SUGAR SCORE CALCULATOR MODULE] Generate Result:\n"+ FinalResultText);
    		    System.out.println("+++++++++++++++++++++++++++++++++ Summary ++++++++++++++++++++++++++++++ : \n");
    		    System.out.println("- FUDGE TRANSCRIPTOR : "+elapsedTime[0]+" ms");
    		    System.out.println("- CRISP Evalutation  : "+elapsedTime[1]+" ms");
    		    System.out.println("- FUDGE Calculator   : "+elapsedTime[2]+"ms");
    		    System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  \n");
    		}
    		/***/
    		//result.setAtsData(SyntacticalAnalyser.Syntaxer.getAtsData(ats));
    		/***/
    	return result;}
    
  /*Method for evaluating a Fuzzy Quantified query :
    - we evaluate Q1 and for each result from Q1, we evaluate Q2 and Q3 and we calculate the final satisfaction degree.
    */
		public CypherResult evaluateFQuantifiedQuery(String query, Neo4jService service, long time, long[] elapsedTime) throws Exception {
		CypherResult result1 = null;
		CypherResult result2 = null;
	   	CypherResult result3 = null;
	   		   	
	   	long[] 	startTime1= new long[3],
    			stopTime1=new long[3],
    			elapsedTime1=new long[3];
    	Q1 q1= new Q1();
		String query1=q1.fudge_transcriptor_module(query);
		String quantif=q1.quantificateur; // Fuzzy quantifier
		double value1=q1.v1; // First value 
		double value2=q1.v2; // Second value
		String firstNode=q1.firstNode;
		result1 = evaluateFUDGEQquery(query1,service,time, elapsedTime);
		String FinalResultText1 = result1.getText();
		 System.out.println("\n[SUGAR SCORE CALCULATOR MODULE] Generate Result of Q1:\n"+ FinalResultText1);
		
		   ArrayList<String> ListNode = new ArrayList<String>();
			for (Map<String, Object> row : result1) {
		    	Node n = (Node) row.get(q1.firstNode);
		                if (!ListNode.contains(n.getProperty("name"))){
		                	ListNode.add((String)n.getProperty("name"));
			            				}
		                
			}
			
    	  	
          Q2 q2	= new Q2();
		  Q3 q3	= new Q3();
		  HashMap quant =new HashMap<String, Double>();	
			for (int k=0; k<ListNode.size(); k++){
				
				String query2=q2.fudge_transcriptor_module(query, ListNode.get(k)); 
				result2=evaluateFUDGEQquery(query2,service,time, elapsedTime);
				String FinalResultText2 = result2.getText();
				 System.out.println("\n[SUGAR SCORE CALCULATOR MODULE] Generate Result of Q2:\n"+ FinalResultText2);
				
				 double degree1 = 0 ;
				 double sum1 = 0;
				 for (Map<String, Object> row : result2) {
					  degree1= (double) row.get("Satisfaction_degree");
					  sum1= sum1+ degree1;
				 }
					 
			String query3=q3.fudge_transcriptor_module(query, ListNode.get(k));     	
			result3=evaluateFUDGEQquery(query3,service,time,elapsedTime);
			String FinalResultText3 = result3.getText();
			 System.out.println("\n[SUGAR SCORE CALCULATOR MODULE] Generate Result of Q3:\n"+ FinalResultText3);
			
			    double degree2 = 0 ;
				double sum2 = 0;
			    for (Map<String, Object> row: result3) {
		              degree2 = (double) row.get("Satisfaction_degree");
			           sum2= sum2 + degree2;
					 }

			   double Finaldegree = sum1 /sum2;	 
		  if (q1.typeQ=="DEFINEQRASC"){
			   IncreasingTrapezoidalFunction function1 = new IncreasingTrapezoidalFunction(quantif, value1, value2);
			   if (!quant.containsKey(ListNode.get(k))){
			          quant.put(ListNode.get(k), function1.mu(Finaldegree));
				 }	
			   }
			   else if (q1.typeQ=="DEFINEQRDESC"){
				   DecreasingTrapezoidalFunction function2 = new DecreasingTrapezoidalFunction(quantif, value1, value2);
				   if (!quant.containsKey(ListNode.get(k))){
				          quant.put(ListNode.get(k), function2.mu(Finaldegree));
					 }	
			   }    
		        }
			
		
					result1.Quant_score_calculator_module(quant);
					 String FinalResultText = result1.getText();
					 System.out.println("\n[SUGAR SCORE CALCULATOR MODULE] Generate Result:\n"+ FinalResultText);
		       
		return result1;
		} 
		
		//Method for evaluating a FUDGE Quantified query : Returning only the first node.
	private CypherResult evaluateFUDGEQquery(String query, Neo4jService service, long time, long[] elapsedTime) throws Exception {	
		CypherResult result = null;
		long[] startTime= new long[3], stopTime=new long[3];
		elapsedTime=new long[3];
        //Analyse Lexicale / generation d'une liste de tokens
        ArrayList<Token> tokens = Lexer.tokenize(query);
        //Analyse Syntaxique, generation de l'arbre syntaxique abstrait
        TokenTree ats = Syntaxer.generateATS(tokens);
        /*System.out.println("tree"+ ats.toString(tokens.size()));*/
        String errorValue=SyntacticalAnalyser.SyntaxException.getErrorValue();
        if(errorValue!="") {
        throw new Exception(errorValue);
        }
        //data.put("ATS",ats.toString(0));	
        query=TranslatorQuant.simpleTranslate(ats);
        System.out.println("\n[SUGAR TRANSCRIPTOR MODULE] Generate Cypher query:\n"+query+"\n");
        
        for (String q : splitQuery(query)) {
        result = service.cypherQuery(q);
        Map <String,TrapezoidalFunction> trapezList = TranslatorQuant.getTrapezList();
        
        // Satisfaction degree calculation
	    result.fudge_score_calculator_module(trapezList);
	}
	/***/
	//result.setAtsData(SyntacticalAnalyser.Syntaxer.getAtsData(ats));
	/***/
return result;}
    
	public boolean fuzzyQuantified(String q){
    	if (q.toUpperCase().contains("HAVING"))
		return true;
    	else return false;   	
    }

	private boolean noViz(Map<String, Object> params) {
        return "none".equals(params.get("viz"));
    }

    private String[] splitQuery(String allQueries) {
        return allQueries.split(";\n");
    }

    private boolean dontInitialize(Neo4jService service) {
        return !service.doesOwnDatabase() || service.isInitialized();
    }

    protected long trace(String msg, long time) {
        long now = System.currentTimeMillis();
        log("## " + msg + " took: " + (now - time) + " ms.");
        return now;
    }


    public String shortenUrl(String uri) {
        try {
            final InputStream stream = (InputStream) new URL("http://tinyurl.com/api-create.php?url=" + URLEncoder.encode(uri, "UTF-8")).getContent();
            final String shortUrl = new Scanner(stream).useDelimiter("\\z").next();
            stream.close();
            return shortUrl;
        } catch (IOException ioe) {
            return null;
        }
    }

    public Map<String, Object> execute(Neo4jService service, GraphInfo info, Map<String,Object> params) {
        final Map<String, Object> result = this.execute(service, info.getInit(), info.getQuery(), info.getVersion(),params);
        result.put("message",info.getMessage());
        return result;
    }

    protected String param(Map input, String param, String defaultValue) {
        if (input==null) return defaultValue;
        String data = (String) input.get(param);
        if (data == null || data.isEmpty()) {
            data = defaultValue;
        } else {
            log(param+": "+data);
        }
        return data;
    }

    public Map<String, Object> init(Neo4jService service, Map<String,Object> input) {
        input.put("init",param(input,"init",DEFAULT_GRAPH_CYPHER));
        input.put("query",param(input,"query",DEFAULT_QUERY));
        return execute(service, GraphInfo.from(input), input);
    }

    protected String baseUri(HttpServletRequest request, String query, final String path) {
        final String requestURI = request.getRequestURL().toString();
        try {
            final URI uri = new URI(requestURI);
            return new URI(uri.getScheme(),null,uri.getHost(),uri.getPort(), path,query,null).toString();
        } catch (URISyntaxException e) {
            throw new RuntimeException("Error parsing URI from "+requestURI+" query "+query);
        }
    }

    protected String encodeParam(String param, String value) {
        if (value==null || value.trim().isEmpty()) return "";
        try {
            return param + "=" + URLEncoder.encode(value, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return param + "=" + URLEncoder.encode(value);
        }
    }
    public String createInitialUri(Request request, GraphInfo info) {
        return baseUri( request.raw(),
			encodeParam("init",  info.getInit()) +
			encodeParam("query", info.getQuery()) +
			encodeParam("version", info.getVersion()) +
			encodeParam("message", info.getMessage()) +
			encodeParam("no_root", info.hasRoot() ? null : "true"),
			null);
    }

    Map<String, Object> init(Neo4jService service, String id, Map<String,Object> params) {
        final GraphInfo info = storage.find(id);
        Map<String, Object> result;
        if (info!=null) {
            result = execute(service, info.getInit(), info.getQuery(), info.getVersion(), params);
            result.put("message",info.getMessage());
        } else {
            result = init(service, params);
            result.put("error","Graph not found for id " + id+ " rendering default");
        }
        return result;
    }

    public Object share(Request request, Map input) {
        final GraphInfo info = GraphInfo.from(input);
        try {
            if (storage!=null) return storage.create(info).getId();
        } catch(Exception e) {
            log("Error storing shared data "+info);
            e.printStackTrace();
        }
        final String uri = createInitialUri(request, info);
        return shortenUrl(uri);
    }

    public void initFromUrl(Neo4jService service, URL url, final String query) {
        if (!service.doesOwnDatabase()) return;
        final String urlString = url.toString().replaceAll("/cypher/?$", "");
        final RestAPI restApi = new RestAPIFacade(urlString);
        final QueryResult<Map<String,Object>> cypherResult = new RestCypherQueryEngine(restApi).query(query, null);
        final SubGraph graph = new SubGraph();
        for (Map<String, Object> row : cypherResult) {
            for (Object value : row.values()) {
                addResultValue(graph, value);
            }
        }
        service.importGraph(graph);
    }

    private void addResultValue(SubGraph subGraph, Object value) {
        if (value instanceof Node) {
            subGraph.add((Node)value);
        }
        if (value instanceof Relationship) {
            subGraph.add((Relationship)value);
        }
        if (value instanceof Iterable) {
            for (Object inner : (Iterable)value) {
                addResultValue(subGraph,inner);
            }
        }
    }

    private <T> T post(URL url, Map<String, Object> data, Class<T> resultType) {
        try {
            final HttpClient client = clientFor(url);
            final PostMethod post = new PostMethod(url.toString());
            post.setDoAuthentication(true);
            post.setRequestHeader("Accept", "application/json;stream=true");
            final Gson gson = new Gson();
            final String postData = gson.toJson(data);
            post.setRequestEntity(new StringRequestEntity(postData, "application/json", "UTF-8"));
            final int status = client.executeMethod(post);
            if (status != 200) throw new RuntimeException("Return Status Code "+post.getStatusCode()+" "+post.getStatusLine());
            return gson.fromJson(post.getResponseBodyAsString(), resultType);
            
        } catch (Exception e) {
            throw new RuntimeException("Error executing request to "+url+" with "+data+":" + e.getMessage());
        }
    }

    private HttpClient clientFor(URL url) {
        final HttpClient client = new HttpClient();
        final String userInfo = url.getUserInfo();
        if (userInfo != null) {
            final String[] usernamePassword = userInfo.split(":");
            client.getState().setCredentials(new AuthScope(url.getHost(), url.getPort()), new UsernamePasswordCredentials(usernamePassword[0], usernamePassword[1]));
        }
        return client;
    }
    public void initFromUrl2(Neo4jService service, URL url, final String query) {
        final Map cypherResult = post(url, map("query", query), Map.class);
        SubGraph graph=SubGraph.fromRaw(cypherResult, false);
        service.importGraph(graph);
      }
}
