'use strict';

var inputeditor;
var visualizer = new GraphVisualization();
var graphgistWindow;

function append(element, text, doHighlight) {
    if (!text) {
        return;
    }
    if (typeof ( text ) == 'object') {
        text = JSON.stringify(text);
    }

    text = "\n" + text;
    if (doHighlight) {
        text = highlight(text);
    }
    element.append($("<span class='textblock'>" + text + "</span>"));
    element.prop("scrollTop", element.prop("scrollHeight") - element.height());
}

function highlight(text) {
    var newtext = "";

    function appendToNewtext(text, typeClass) {
        if (typeClass == null) {
            newtext = newtext + text;
        }
        else {
            newtext = newtext + "<span class=\"cm-" + typeClass + "\">" + text + "</span>";
        }
    }

    CodeMirror.runMode(text, "cypher", appendToNewtext);
    return newtext;
}

function animateDiv(divId,colorValue) {
	$("#"+divId).stop();
	$("#"+divId).css("border-left-color",colorValue);
	$("#"+divId).animate({'borderLeftColor':'#eeeeee'},3000);
}

function animationError(divId) {
	animateDiv(divId,"red");
}

function animationOk(divId) {
	animateDiv(divId,"green");
}

function post(uri, data, done, dataType) {
	var atsData="";
    data = data.trim();
    // console.log("Post data: " + data);
    $.ajax(uri, {
        type: "POST",
        data: data,
        dataType: dataType || "text",
        beforeSend: setSessionHeader,
        success: function (data) {
			console.log("dataError: " + data["error"]);
			if(typeof data["error"] !='undefined' && data["error"]!="") {
				$('#infoSyntaxError').html(data["error"]);
				data["error"]="";
				animationError("infoSyntaxError");
			} else {
				$('#infoSyntaxError').html("<b>No Syntax Error</b>");
				animationOk("infoSyntaxError");
			}
			
			if(typeof data["atsData"] !='undefined' && data["atsData"]!="") {
				atsData=data["atsData"];
				//generateDiagram(atsData);
			} 
			
            if (dataType == "text") {
                append($("#output"), data);
            }
            if (done) {
                done(data);
            }
        },
        error: function (data, error) {
            append($("#output"), "Error: " + error + "\n" + data.responseText + "");
        }
    });
}

function getParameters() {
    var pairs = window.location.search.substr(1).split('&');
    if (!pairs) {
        return {};
    }
    var result = {};
    for (var i = 0; i < pairs.length; ++i) {
        var pair = pairs[i].split('=');
        //console.log( pair );
        if (pair.length != 2) {
            continue;
        }
        result[pair[0]] = decodeURIComponent(pair[1].replace(/\+/g, " "));
    }
    return result;
}

function share(fn) {
    $.ajax("console/url", {
        type: "GET",
        success: fn
    });
}

function isCypher(query) {
    return query && query.match(/\b(start|merge|match|foreach|drop|create|delete|relate|return)\b/i);
}

function viz(data) {
    if (!vizVisible) {
        return;
    }
    var output = $("#output");
    output.children('#graph').remove();
    if (data) {
        var h = output.height();
        visualizer.visualize("output", output.width(), h, data);
    }
    else {
        var query = getQuery();
        if (!isCypher(query)) {
            query = "";
        }
        $.ajax("console/visualization?query=" + encodeURIComponent(query), {
            type: "GET",
            beforeSend: setSessionHeader,
            dataType: "json",
            success: function (data) {
                visualizer.visualize("output", output.width(), output.height(),data)
            }
        });
    }
    // graph.show();
}

function setSessionHeader(request) {
    request.setRequestHeader("X-Session", session_id)
}
function reset(done) {
    $.ajax("console", {
        beforeSend: setSessionHeader,
        type: "DELETE",
        success: done
    });
    return false;
}

function share_yuml(query) {
    if (!query) {
        return;
    }
    $.ajax("console/to_yuml?query=" + encodeURIComponent(query), {
        type: "GET",
        beforeSend: setSessionHeader,
        dataType: "text",
        success: function (data) {
            $('#share_yuml').attr("href", data);
        }
    });
}

function base_url() {
    return document.location.protocol + "//" + document.location.host;
}

function store_graph_info() {
    var init = $('#share_init').val();
    var query = $('#share_query').val();
    query = isCypher(query) ? query : null;
    var version = $('#share_version').val();
    var no_root = $("#share_no_root").is(":checked");
    var id = $('#share_short').val();
    if (id && ( id.length == 0 || id.match("http://.+") )) {
        id = null;
    }
    var message = null;
    // var uri =
    // base_url()+'?init='+encodeURIComponent(init)+'&query='+encodeURIComponent(query)+'&version='+encodeURIComponent(version);
    // if (no_root) uri+="&no_root=true";

    $.ajax("/r/share", {
        type: "POST",
        dataType: "text",
        contentType: "application/json",
        data: JSON.stringify({
            id: id,
            init: init,
            query: query,
            version: version,
            no_root: no_root,
            message: message
        }),
        success: function (data) {
            var uri = ( data.indexOf("http") != 0 ) ? base_url() + "/r/" + data : data;
            // console.log(uri);
            var frame = '<iframe width="600" height="300" src="' + uri + '"/>';
            $('#share_iFrame').val(frame);
            $('#share_url').attr("href", uri);
            $('#share_short').val(uri);
            addthis.update('share', 'url', uri);
        }
    });
    share_yuml(query);
    addthis.update('share', 'title', 'Look at this #Neo4j graph: ');
}

var vizVisible = true;
function toggleGraph() {
    vizVisible = !vizVisible;
    if (vizVisible) {
        $('#graph').show();
        viz();
    } else {
        $('#graph').hide();
    }
}

function toggleShare() {
    $.ajax("console/to_cypher", {
        type: "GET",
        beforeSend: setSessionHeader,
        dataType: "text",
        success: function (data) {
            $('#share_init').val(data);
            var query = getQuery();
            query = isCypher(query) ? query : null;
            $('#share_query').val(query);
            $('#shareUrl').toggle();
            share_yuml(query);
        },
        error: function (data, error) {
            append($("#output"), "Error: " + error + "\n" + data.responseText + "");
        }
    });
}

function export_graph(format) {
    $.ajax("console/to_" + format, {
        type: "GET",
        dataType: "text",
        beforeSend: setSessionHeader,
        success: function (data) {
            $('#share_init').val(data);
        },
        error: function (data, error) {
            $('#share_init').val("Error: " + error + "\n" + data.responseText + "");
        }
    });
}

function readable(prefix, map) {
    var col = [];
    for (var key in map) {
        if (map.hasOwnProperty(key) && map[key]) {
            col.push(map[key] + " " + key + ( map[key] > 1 ? "s" : "" ));
        }
    }
    if (!col.length) {
        return "";
    }
    if (col.length == 1) {
        return prefix + " " + col[0] + " ";
    }
    return prefix + " " + col.slice(0, -1).join(", ") + " and " + col[col.length - 1] + " ";
}

function computeInfo(data) {
    var stats = data.stats;
    var info = "Query took " + stats.time + " ms";
    var count = data.json ? stats.rows : 0;
    info += " and returned " + ( count ? count : "no" ) + " rows. ";
    if (stats.containsUpdates) {
        var updates = "\nUpdated the graph - " + readable("created", {
            node: stats.nodesCreated,
            relationship: stats.relationshipsCreated
        }) + readable("deleted", {
            node: stats.nodesDeleted,
            relationship: stats.relationshipsDeleted
        }) + readable("set", {
            property: stats.propertiesSet
        }).replace("propertys", "properties");
        info += updates;
    }
    return info;
}

function inputQuery(query) {
    inputeditor.setValue(query); // .replace(/\n/g, ' ').trim()
    CodeMirror.commands["selectAll"](inputeditor);
    autoFormatSelection(inputeditor);
    resizeOutput();
}

function infoImage(data) {
    var text = "";
    for (var title in data) {
        if (data.hasOwnProperty(title)) {
            text += "<h4>" + title + "</h4>" + data[title] + "\n";
        }
    }
    return $('<button class="btn_pretty">Result Details</button>').attr('title', text).click(function () {
        $('#stats-output').html(text);
        $('#stats').show();
    });
}
function showResults(data) {
    if (data["init"]) {
        append($("#output"), "Graph Setup:");
        append($("#output"), data["init"], true);
    }
    if (data["query"]) {
        append($("#output"), "\nQuery:");
        inputQuery(data["query"]);
        append($("#output"), inputeditor.getValue(), true);
        resizeOutput();
    }
    if (data["result"]) {
        append($("#output"), "\n");
        renderResult("output", data);
        append($("#output"), computeInfo(data));
        $("#output").append(infoImage({
            "Query Results": data.result,
            "Execution Plan": data.plan
        }));
        append($("#output"), "\n");
    }
    if (data["error"]) {
        append($("#output"), "Error: " + data["error"], true);
    }
    if (data["visualization"]) {
        viz(data.visualization);
    }
}

function send(query) {
    if (isCypher(query)) {
        post("console/cypher"+getPostQueryParams(), query, showResults, "json");
    }
    else {
        post("console/geoff", query, function () {
            viz();
        });
    }
}

function showVersion(json) {
    $('#version').val(json['version']);
    $('#share_version').val(json['version']);
}

function welcome_msg() {
    return $("#welcome_msg").html();
}

function showWelcome(json) {
    var output = $("#output");

    if (json['message']) {
        if (json['message'] != "none") {
            output.append(json['message']);
        }
    }
    else {
        output.append(welcome_msg());
    }
}

function resizeOutput() {
    var $output = $("#output");
    var editorHeight = $(".CodeMirror-scroll").height();
    var windowHeight = $(window).height();
    $output.css({
        'height': ( windowHeight - editorHeight - 15 ) + 'px'
    });
    $output.animate({
        scrollTop: $output.prop("scrollHeight")
    }, 10);
    $("#query_button").css({
        'height': editorHeight + 'px'
    });
}

function getSelectedRange(editor) {
    return {
        from: editor.getCursor(true),
        to: editor.getCursor(false)
    };
}

function autoFormatSelection(editor) {
    var range = getSelectedRange(editor);
    editor.autoFormatRange(range.from, range.to);
}

function getQuery() {
    return inputeditor.getValue();
}

function query() {
    inputQuery(getQuery());
    send(getQuery());
}

function cleanDb() {
    if ($('#version').val()=="1.9")
        send("START n=node(*) MATCH (n)-[r?]-() DELETE n, r");
    else
        send("MATCH (n) OPTIONAL MATCH (n)-[r]-() DELETE n, r");
}

function parseMessage(data) {
    var first = data.trim().charAt(0);
    if (first == '{' || first == '[') {
        var result = JSON.parse(data);
        if (result.data && typeof(result.data) == "string") result.data = [result.data];
        return  result;
    }
    return {data: [data], action: "query"};
}

function handleMessage(msg) {
    console.log("msg",msg);
    if (msg.action == "init") {
        sendInit(msg.data, msg.call_id);
        // send init with data
        return;
    }
    if (msg.action == "query" || msg.action == "input") {
        sendNext(msg);
    }
}

function getPostQueryParams() {
    return vizVisible ? "" : "?viz=none";
}

function sendNext(msg) {
    if (msg.data.length == 0) return;
    var _query = msg.data.shift();
    if (msg.data.length == 0 && msg.action == "input") {
        inputQuery(_query)
        return;
    }
    post("console/cypher"+getPostQueryParams(), _query, function (res) {
        if (msg.action === "query" && msg.call_id && graphgistWindow) {
            res.call_id = msg.call_id;
            console.log("res",res);
            graphgistWindow.postMessage(JSON.stringify(res), "*");
        }
        else {
            showResults(res);
        }
        sendNext(msg);
    }, "json");
}

function close(id) {
    $(id).hide().children("iframe").removeAttr("src");
}

function sendInit(params, callId) {
    post("console/init", JSON.stringify(params), function (json) {
        showResults(json);
        showVersion(json);
        showWelcome(json);
        if (callId && graphgistWindow) {
            json.call_id = callId;
            graphgistWindow.postMessage(JSON.stringify(json), "*");
        }
    }, "json", callId);
}


function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
        .toString(16)
        .substring(1);
};

function guid() {
    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
        s4() + '-' + s4() + s4() + s4();
}

var session_id=guid();

$(document).ready(
    function () {
        inputeditor = CodeMirror.fromTextArea(document.getElementById("input"), {
            lineNumbers: false,
            readOnly: false,
            mode: "cypher",
            theme: "neo",
            onKeyEvent: function (inputeditor, e) {
                if (e.type == 'keydown') {
                    // resize output while typing...
                    resizeOutput();
                    if (e.which == 13 && !e.shiftKey) {
                        query();
                        // cancel normal enter (must type shift enter to add lines)
                        e.preventDefault();
                        return true;
                    }
                    // allow the rest to go through.
                    return false;
                }
            }
        });
        // set id to align with CSS from Neo4j Browser
        $("#form > div.CodeMirror").first().attr("id", "editor");
        // resize output if window gets resized
        $(window).resize(resizeOutput);

        window.addEventListener("message", function (e) {
            if (e.origin.match(/addthis|cloudfront.net/)) {
                return;
            }
            //console.log( "postMessage", e );
            graphgistWindow = e.source;
            var msg = parseMessage(e.data);
            handleMessage(msg);
        });

//        if (!session) {
            var params = getParameters();
            if (params['viz']=="none") {
                vizVisible=false;
            }
            sendInit(params);
//        }
//        else {
//            query();
//        }

        $('#version').change(function () {
            post("console/version", $('#version').val(), showVersion, "json");
            $(this).hide();
        });
        var isInIFrame = window.location != window.parent.location;
        if (!isInIFrame) {
            inputeditor.focus();
        }

        $("body").keyup(function (e) {
            if (e.which == 27) {
                close(".popup");
            }
            return true;
        });
        $("#video").click(
            function () {
                close("#info");
                var url = $(this).attr("video") + "?badge=0&title=0&portrait=0&autoplay=1&rel=0&byline=0";
                $("#player").show();
                $("#player iframe").attr("width", $("#player").width()).attr("height", $("#player").height())
                    .attr("src", url);
            });
        $(".popup .btn_close").click(function () {
            close($(this).parent());
        });
        $("code[type=cypher]").each(function() {
	       var text=highlight($(this).text());
	       $(this).html(text);
		});
    });

	
//** legend **

function drawLegend() {
	var legendRectSize=18,
	legendSpacing=4;

	var color = d3.scale.ordinal()
		.domain(["AbstractToken","CYPHER Keyword","FUDGE Keyword","Identifier or Number","Punctuation", "Operator","Unknown","ErrorToken"])
		.range(["lightblue","lightblue","violet","lightgreen","lightgrey", "lightcoral","#FAA689", "orange"]);
		
	var color2 = d3.scale.ordinal()
		.domain(["AbstractToken","CYPHER Keyword","FUDGE Keyword","Identifier or Number","Punctuation", "Operator","Unknown","ErrorToken"])
		.range(["black","#1d75b3","purple","green", "grey", "maroon","#FA4002","black"]);
		
	var legend = d3.select('svg')
		.append("g")
		.selectAll("g")
		.data(color.domain())
		.enter()
		.append('g')
		  .attr('class', 'legend')
		  .attr('transform', function(d, i) {
			var height = legendRectSize+10;
			var x = 1;
			var y = 1+i * height;
			return 'translate(' + x + ',' + y + ')';
		});

	legend.append('rect')
		.attr('width', legendRectSize)
		.attr('height', legendRectSize)
		.style('fill', color)
		.style('stroke', color2)
		.style('stroke-width',2);

	legend.append('text')
		.attr('x', legendRectSize + legendSpacing)
		.attr('y', legendRectSize - legendSpacing)
		.text(function(d) { return d; });
}

function generateDiagram(atsData) {
	// ************** Generate the tree diagram	 *****************
	var margin = {top: 20, right: 120, bottom: 20, left: 120},
	width = 5000 - margin.right - margin.left,
	height = 600 - margin.top - margin.bottom;
	
	var i = 0,
		duration = 750,
		root;

	var tree = d3.layout.tree()
		.size([height, width]);

	var diagonal = d3.svg.diagonal()
		.projection(function(d) { return [d.y, d.x]; });

	var svg = d3.select("body").append("svg")
		.attr("width", width + margin.right + margin.left)
		.attr("height", height + margin.top + margin.bottom)
	  .append("g")
		.attr("transform", "translate(" + margin.left + "," + margin.top + ")");
		
	d3.json("data.json", function (json) {
		root = JSON.parse(atsData);
		root.x0 = height/2;
		root.y0 = 0;
		update(root);
		drawLegend();
	});
	 
	d3.select(self.frameElement).style("height", "500px");
}

function update(source) {

  // Compute the new tree layout.
  var nodes = tree.nodes(root).reverse(),
	  links = tree.links(nodes);

  // Normalize for fixed-depth.
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  // Update the nodes…
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

  // Enter any new nodes at the parent's previous position.
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; })
	  .on("click", click);

  nodeEnter.append("circle")
	  .attr("r", function(d){ return d.value; })
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });

  nodeEnter.append("text")
	  .attr("x", function(d) { return ((d.children || d._children) || d.level == "orange" ) ? (d.value + 4) * -1 : d.value + 4 })
	  .attr("y", function(d) { return ((d.children || d._children) || d.level == "orange" )  ? (d.value + 4) * -1 : 0 })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return ((d.children || d._children) || d.level == "orange" ) ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);

  // Transition nodes to their new position.
  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  nodeUpdate.select("circle")
	  .attr("r", 10)
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });
	

  nodeUpdate.select("text")
	  .style("fill-opacity", 1)
	  .style("fill", function(d){ return d.type; });

  // Transition exiting nodes to the parent's new position.
  var nodeExit = node.exit().transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + source.y + "," + source.x + ")"; })
	  .remove();

  nodeExit.select("circle")
	  .attr("r", 1e-6)
	  .style("stroke", function(d){ return d.type; })
	  .style("fill", function(d){ return d.level; });
 
  nodeExit.select("text")
	  .style("fill-opacity", 1e-6);

  // Update the links…
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

  // Enter any new links at the parent's previous position.
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .style("stroke", function (d) { return d.target.level;})
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

  // Transition links to their new position.
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);

  // Transition exiting nodes to the parent's new position.
  link.exit().transition()
	  .duration(duration)
	  .attr("d", function(d) {
		var o = {x: source.x, y: source.y};
		return diagonal({source: o, target: o});
	  })
	  .remove();

  // Stash the old positions for transition.
  nodes.forEach(function(d) {
	d.x0 = d.x;
	d.y0 = d.y;
  });
}

// Toggle children on click.
function click(d) {
  if (d.children) {
	d._children = d.children;
	d.children = null;
  } else {
	d.children = d._children;
	d._children = null;
  }
  update(d);
  drawLegend();
}
